@@ -19,9 +19,9 @@ from string import join, capitalize, lower
 from zope.interface import implements
 from twisted.internet import protocol, reactor
 from twisted.words.protocols import irc
-from twisted.python import log, failure
+from twisted.python import usage, log
 from twisted.application import internet
-from twisted.internet import task
+from twisted.internet import defer, task
 
 from buildbot import interfaces, util
 from buildbot import version
@@ -29,7 +29,6 @@ from buildbot.interfaces import IStatusReceiver
 from buildbot.sourcestamp import SourceStamp
 from buildbot.status import base
 from buildbot.status.results import SUCCESS, WARNINGS, FAILURE, EXCEPTION, RETRY
-from buildbot.scripts.runner import ForceOptions
 from buildbot.process.properties import Properties
 
 # twisted.internet.ssl requires PyOpenSSL, so be resilient if it's missing
@@ -68,6 +67,30 @@ class UsageError(ValueError):
     def __init__(self, string = "Invalid usage", *more):
         ValueError.__init__(self, string, *more)
 
+class ForceOptions(usage.Options):
+    optParameters = [
+        ["builder", None, None, "which Builder to start"],
+        ["branch", None, None, "which branch to build"],
+        ["revision", None, None, "which revision to build"],
+        ["reason", None, None, "the reason for starting the build"],
+        ["props", None, None,
+         "A set of properties made available in the build environment, "
+         "format is --properties=prop1=value1,prop2=value2,.. "
+         "option can be specified multiple times."],
+        ]
+
+    def parseArgs(self, *args):
+        args = list(args)
+        if len(args) > 0:
+            if self['builder'] is not None:
+                raise UsageError("--builder provided in two ways")
+            self['builder'] = args.pop(0)
+        if len(args) > 0:
+            if self['reason'] is not None:
+                raise UsageError("--reason provided in two ways")
+            self['reason'] = " ".join(args)
+
+
 class IrcBuildRequest:
     hasStarted = False
     timer = None
@@ -703,7 +726,7 @@ class IRCContact(base.StatusReceiver):
 
     def act(self, action):
         if not self.muted:
-            self.bot.me(self.dest, action.encode("ascii", "replace"))
+            self.bot.describe(self.dest, action.encode("ascii", "replace"))
 
     # main dispatchers for incoming messages
 
@@ -720,7 +743,8 @@ class IRCContact(base.StatusReceiver):
         # single user.
         message = message.lstrip()
         if self.silly.has_key(message):
-            return self.doSilly(message)
+            self.doSilly(message)
+            return defer.succeed(None)
 
         parts = message.split(' ', 1)
         if len(parts) == 1:
@@ -731,31 +755,28 @@ class IRCContact(base.StatusReceiver):
         meth = self.getCommandMethod(cmd)
         if not meth and message[-1] == '!':
             self.send("What you say!")
-            return
+            return defer.succeed(None)
 
-        error = None
-        try:
-            if meth:
-                meth(args.strip(), who)
-        except UsageError, e:
-            self.send(str(e))
-        except:
-            f = failure.Failure()
-            log.err(f)
-            error = "Something bad happened (see logs)"
-
-        if error:
-            try:
-                self.send(error)
-            except:
-                log.err()
+        if meth:
+            d = defer.maybeDeferred(meth, args.strip(), who)
+            @d.addErrback
+            def usageError(f):
+                f.trap(UsageError)
+                self.send(str(f.value))
+            @d.addErrback
+            def logErr(f):
+                log.err(f)
+                self.send("Something bad happened (see logs)")
+            d.addErrback(log.err)
+            return d
+        return defer.succeed(None)
 
     def handleAction(self, data, user):
         # this is sent when somebody performs an action that mentions the
         # buildbot (like '/me kicks buildbot'). 'user' is the name/nick/id of
         # the person who performed the action, so if their action provokes a
         # response, they can be named.  This is 100% silly.
-        if not data.endswith("s buildbot"):
+        if not data.endswith("s "+ self.bot.nickname):
             return
         words = data.split()
         verb = words[-2]
@@ -806,6 +827,7 @@ class IrcStatusBot(irc.IRCClient):
             self.msg(dest, message)
 
     def getContact(self, name):
+        name = name.lower() # nicknames and channel names are case insensitive
         if name in self.contacts:
             return self.contacts[name]
         new_contact = self.contactClass(self, name)
@@ -821,7 +843,6 @@ class IrcStatusBot(irc.IRCClient):
     def privmsg(self, user, channel, message):
         user = user.split('!', 1)[0] # rest is ~user@hostname
         # channel is '#twisted' or 'buildbot' (for private messages)
-        channel = channel.lower()
         if channel == self.nickname:
             # private message
             contact = self.getContact(user)
@@ -838,7 +859,7 @@ class IrcStatusBot(irc.IRCClient):
         user = user.split('!', 1)[0] # rest is ~user@hostname
         # somebody did an action (/me actions) in the broadcast channel
         contact = self.getContact(channel)
-        if "buildbot" in data:
+        if self.nickname in data:
             contact.handleAction(data, user)
 
     def signedOn(self):
