@@ -27,58 +27,86 @@ from mock import Mock
 
 class FakeChange:
     properties = Properties()
-    who = "me"
-
+    def __init__(self, number = None):
+        self.number = number
+        self.who = "me"
+        
 class FakeSource:
-    changes = [FakeChange()]
-    branch = None
-    revision = "12345"
-    repository = None
-    project = None
-    patch_info = (None, None)
-    patch = None
+    def __init__(self):
+        self.sourcestampsetid = None
+        self.changes = []
+        self.branch = None
+        self.revision = None
+        self.repository = ''
+        self.codebase = ''
+        self.project = ''
+        self.patch_info = None
+        self.patch = None
+
+    def getRepository(self):
+        return self.repository
 
 class FakeRequest:
-    source = FakeSource()
-    reason = "Because"
-    properties = Properties()
+    def __init__(self):
+        self.sources = []
+        self.reason = "Because"
+        self.properties = Properties()
 
-    def mergeWith(self, others):
-        return self.source
+    def mergeSourceStampsWith(self, others):
+        return self.sources
 
     def mergeReasons(self, others):
         return self.reason
 
 class FakeBuildStep:
-    haltOnFailure = False
-    flunkOnWarnings = False
-    flunkOnFailure = True
-    warnOnWarnings = True
-    warnOnFailure = False
-    alwaysRun = False
-    name = 'fake'
+    def __init__(self):
+        self.haltOnFailure = False
+        self.flunkOnWarnings = False
+        self.flunkOnFailure = True
+        self.warnOnWarnings = True
+        self.warnOnFailure = False
+        self.alwaysRun = False
+        self.name = 'fake'
 
 class FakeMaster:
-    locks = {}
-    parent = Mock()
+    def __init__(self):
+        self.locks = {}
+        self.parent = Mock()
+        
     def getLockByID(self, lockid):
         if not lockid in self.locks:
             self.locks[lockid] = lockid.lockClass(lockid)
         return self.locks[lockid]
 
 class FakeBuildStatus(Mock):
-    implements(interfaces.IProperties)
-
+    implements(interfaces.IProperties)   
+        
 class FakeBuilderStatus:
     implements(interfaces.IBuilderStatus)
 
+class FakeStepFactory(object):
+    """Fake step factory that just returns a fixed step object."""
+    implements(interfaces.IBuildStepFactory)
+    def __init__(self, step):
+        self.step = step
+
+    def buildStep(self):
+        return self.step
+
 class TestBuild(unittest.TestCase):
 
     def setUp(self):
         r = FakeRequest()
+        r.sources = [FakeSource()]
+        r.sources[0].changes = [FakeChange()]
+        r.sources[0].revision = "12345"
+
+        self.request = r
+        self.master = FakeMaster()
+
         self.build = Build([r])
         self.builder = Mock()
-        self.builder.botmaster = FakeMaster()
+        self.builder.botmaster = self.master
         self.build.setBuilder(self.builder)
 
     def testRunSuccessfulBuild(self):
@@ -87,7 +115,7 @@ class TestBuild(unittest.TestCase):
         step = Mock()
         step.return_value = step
         step.startStep.return_value = SUCCESS
-        b.setStepFactories([(step, {})])
+        b.setStepFactories([FakeStepFactory(step)])
 
         slavebuilder = Mock()
 
@@ -102,7 +130,7 @@ class TestBuild(unittest.TestCase):
 
         step = Mock()
         step.return_value = step
-        b.setStepFactories([(step, {})])
+        b.setStepFactories([FakeStepFactory(step)])
 
         slavebuilder = Mock()
 
@@ -133,8 +161,8 @@ class TestBuild(unittest.TestCase):
         step2.return_value = step2
         step2.alwaysRun = True
         b.setStepFactories([
-            (step1, {}),
-            (step2, {}),
+            FakeStepFactory(step1),
+            FakeStepFactory(step2),
             ])
 
         slavebuilder = Mock()
@@ -170,7 +198,7 @@ class TestBuild(unittest.TestCase):
         claimCount = [0]
         lock_access = l.access('counting')
         l.access = lambda mode: lock_access
-        real_lock = b.builder.botmaster.getLockByID(l).getLock(slavebuilder)
+        real_lock = b.builder.botmaster.getLockByID(l).getLock(slavebuilder.slave)
         def claim(owner, access):
             claimCount[0] += 1
             return real_lock.old_claim(owner, access)
@@ -181,7 +209,7 @@ class TestBuild(unittest.TestCase):
         step = Mock()
         step.return_value = step
         step.startStep.return_value = SUCCESS
-        b.setStepFactories([(step, {})])
+        b.setStepFactories([FakeStepFactory(step)])
 
         b.startBuild(FakeBuildStatus(), None, slavebuilder)
 
@@ -190,6 +218,58 @@ class TestBuild(unittest.TestCase):
                                 in step.method_calls)
         self.assertEquals(claimCount[0], 1)
 
+    def testBuildLocksOrder(self):
+        """Test that locks are acquired in FIFO order; specifically that
+        counting locks cannot jump ahead of exclusive locks"""
+        eBuild = self.build
+
+        cBuild = Build([self.request])
+        cBuilder = Mock()
+        cBuilder.botmaster = self.master
+        cBuild.setBuilder(cBuilder)
+
+        eSlavebuilder = Mock()
+        cSlavebuilder = Mock()
+
+        slave = eSlavebuilder.slave
+        cSlavebuilder.slave = slave
+
+        l = SlaveLock('lock', 2)
+        claimLog = []
+        realLock = self.master.getLockByID(l).getLock(slave)
+        def claim(owner, access):
+            claimLog.append(owner)
+            return realLock.oldClaim(owner, access)
+        realLock.oldClaim = realLock.claim
+        realLock.claim = claim
+
+        eBuild.setLocks([l.access('exclusive')])
+        cBuild.setLocks([l.access('counting')])
+
+        fakeBuild = Mock()
+        fakeBuildAccess = l.access('counting')
+        realLock.claim(fakeBuild, fakeBuildAccess)
+
+        step = Mock()
+        step.return_value = step
+        step.startStep.return_value = SUCCESS
+        eBuild.setStepFactories([FakeStepFactory(step)])
+        cBuild.setStepFactories([FakeStepFactory(step)])
+
+        e = eBuild.startBuild(FakeBuildStatus(), None, eSlavebuilder)
+        c = cBuild.startBuild(FakeBuildStatus(), None, cSlavebuilder)
+        d = defer.DeferredList([e, c])
+
+        realLock.release(fakeBuild, fakeBuildAccess)
+
+        def check(ign):
+            self.assertEqual(eBuild.result, SUCCESS)
+            self.assertEqual(cBuild.result, SUCCESS)
+            self.assertEquals(claimLog, [fakeBuild, eBuild, cBuild])
+
+        d.addCallback(check)
+        return d
+
     def testBuildWaitingForLocks(self):
         b = self.build
 
@@ -199,7 +279,7 @@ class TestBuild(unittest.TestCase):
         claimCount = [0]
         lock_access = l.access('counting')
         l.access = lambda mode: lock_access
-        real_lock = b.builder.botmaster.getLockByID(l).getLock(slavebuilder)
+        real_lock = b.builder.botmaster.getLockByID(l).getLock(slavebuilder.slave)
         def claim(owner, access):
             claimCount[0] += 1
             return real_lock.old_claim(owner, access)
@@ -210,7 +290,7 @@ class TestBuild(unittest.TestCase):
         step = Mock()
         step.return_value = step
         step.startStep.return_value = SUCCESS
-        b.setStepFactories([(step, {})])
+        b.setStepFactories([FakeStepFactory(step)])
 
         real_lock.claim(Mock(), l.access('counting'))
 
@@ -237,7 +317,7 @@ class TestBuild(unittest.TestCase):
         step.return_value = step
         step.startStep.return_value = SUCCESS
         step.alwaysRun = False
-        b.setStepFactories([(step, {})])
+        b.setStepFactories([FakeStepFactory(step)])
 
         real_lock.claim(Mock(), l.access('counting'))
 
@@ -270,7 +350,7 @@ class TestBuild(unittest.TestCase):
         step.return_value = step
         step.startStep.return_value = SUCCESS
         step.alwaysRun = False
-        b.setStepFactories([(step, {})])
+        b.setStepFactories([FakeStepFactory(step)])
 
         real_lock.claim(Mock(), l.access('counting'))
 
@@ -301,9 +381,7 @@ class TestBuild(unittest.TestCase):
         real_lock = b.builder.botmaster.getLockByID(l).getLock(slavebuilder)
 
         step = LoggingBuildStep(locks=[lock_access])
-        def factory(*args):
-            return step
-        b.setStepFactories([(factory, {})])
+        b.setStepFactories([FakeStepFactory(step)])
 
         real_lock.claim(Mock(), l.access('counting'))
 
@@ -473,7 +551,185 @@ class TestBuild(unittest.TestCase):
         self.assertEqual(terminate, True)
         self.assertEqual(b.result, RETRY)
 
+class TestMultipleSourceStamps(unittest.TestCase):
 
+    def setUp(self):
+        r = FakeRequest()
+        s1 = FakeSource()
+        s1.repository = "repoA"
+        s1.codebase = "A"
+        s1.changes = [FakeChange(10), FakeChange(11)]
+        s1.revision = "12345"
+        s2 = FakeSource()
+        s2.repository = "repoB"
+        s2.codebase = "B"
+        s2.changes = [FakeChange(12),FakeChange(13)]
+        s2.revision = "67890"
+        s3 = FakeSource()
+        s3.repository = "repoC"
+        # no codebase defined
+        s3.changes = [FakeChange(14),FakeChange(15)]
+        s3.revision = "111213"
+        r.sources.extend([s1,s2,s3])
+        
+        self.build = Build([r])
+
+    def test_buildReturnSourceStamp(self):
+        """
+        Test that a build returns the correct sourcestamp
+        """
+        source1 = self.build.getSourceStamp("A")
+        source2 = self.build.getSourceStamp("B")
+
+        self.assertEqual( [source1.repository, source1.revision], ["repoA", "12345"])
+        self.assertEqual( [source2.repository, source2.revision], ["repoB", "67890"])
+
+    def test_buildReturnSourceStamp_empty_codebase(self):
+        """
+        Test that a build returns the correct sourcestamp if codebase is empty
+        """
+        codebase = ''
+        source3 = self.build.getSourceStamp(codebase)
+        self.assertTrue(source3 is not None)
+        self.assertEqual( [source3.repository, source3.revision], ["repoC", "111213"])
+        
+
+class TestBuildBlameList(unittest.TestCase):
+
+    def setUp(self):
+        self.sourceByMe = FakeSource()
+        self.sourceByMe.repository = "repoA"
+        self.sourceByMe.codebase = "A"
+        self.sourceByMe.changes = [FakeChange(10), FakeChange(11)]
+        self.sourceByMe.changes[0].who = "me"
+        self.sourceByMe.changes[1].who = "me"
+
+        self.sourceByHim = FakeSource()
+        self.sourceByHim.repository = "repoB"
+        self.sourceByHim.codebase = "B"
+        self.sourceByHim.changes = [FakeChange(12), FakeChange(13)]
+        self.sourceByHim.changes[0].who = "him"
+        self.sourceByHim.changes[1].who = "him"
+
+        self.patchSource = FakeSource()
+        self.patchSource.repository = "repoB"
+        self.patchSource.codebase = "B"
+        self.patchSource.changes = []
+        self.patchSource.revision = "67890"
+        self.patchSource.patch_info = ("jeff", "jeff's new feature")
+
+    def test_blamelist_for_changes(self):
+        r = FakeRequest()
+        r.sources.extend([self.sourceByMe, self.sourceByHim])
+        build = Build([r])
+        blamelist = build.blamelist()
+        self.assertEqual(blamelist, ['him', 'me'])
+
+    def test_blamelist_for_patch(self):
+        r = FakeRequest()
+        r.sources.extend([self.patchSource])
+        build = Build([r])
+        blamelist = build.blamelist()
+        self.assertEqual(blamelist, ['jeff'])
+
+class TestSetupProperties_MultipleSources(unittest.TestCase):
+    """
+    Test that the property values, based on the available requests, are 
+    initialized properly
+    """
+    def setUp(self):
+        self.props = {}
+        r = FakeRequest()
+        r.sources = []
+        r.sources.append(FakeSource())
+        r.sources[0].changes = [FakeChange()]
+        r.sources[0].repository = "http://svn-repo-A"
+        r.sources[0].codebase = "A"
+        r.sources[0].branch = "develop"
+        r.sources[0].revision = "12345"
+        r.sources.append(FakeSource())
+        r.sources[1].changes = [FakeChange()]
+        r.sources[1].repository = "http://svn-repo-B"
+        r.sources[1].codebase = "B"
+        r.sources[1].revision = "34567"
+        self.build = Build([r])
+        self.build.setStepFactories([])
+        self.builder = Mock()
+        self.build.setBuilder(self.builder)
+        self.build.build_status = FakeBuildStatus()
+        # record properties that will be set
+        self.build.build_status.setProperty = self.setProperty
+
+    def setProperty(self, n,v,s, runtime = False):
+        if s not in self.props:
+            self.props[s] = {}
+        if not self.props[s]:
+            self.props[s] = {}
+        self.props[s][n] = v
+        
+    def test_sourcestamp_properties_not_set(self):
+        self.build.setupProperties()
+        self.assertTrue("codebase" not in self.props["Build"])
+        self.assertTrue("revision" not in self.props["Build"])
+        self.assertTrue("branch" not in self.props["Build"])
+        self.assertTrue("project" not in self.props["Build"])
+        self.assertTrue("repository" not in self.props["Build"])
+
+class TestSetupProperties_SingleSource(unittest.TestCase):
+    """
+    Test that the property values, based on the available requests, are 
+    initialized properly
+    """
+    def setUp(self):
+        self.props = {}
+        r = FakeRequest()
+        r.sources = []
+        r.sources.append(FakeSource())
+        r.sources[0].changes = [FakeChange()]
+        r.sources[0].repository = "http://svn-repo-A"
+        r.sources[0].codebase = "A"
+        r.sources[0].branch = "develop"
+        r.sources[0].revision = "12345"
+        self.build = Build([r])
+        self.build.setStepFactories([])
+        self.builder = Mock()
+        self.build.setBuilder(self.builder)
+        self.build.build_status = FakeBuildStatus()
+        # record properties that will be set
+        self.build.build_status.setProperty = self.setProperty
+
+    def setProperty(self, n,v,s, runtime = False):
+        if s not in self.props:
+            self.props[s] = {}
+        if not self.props[s]:
+            self.props[s] = {}
+        self.props[s][n] = v
+
+    def test_properties_codebase(self):
+        self.build.setupProperties()
+        codebase = self.props["Build"]["codebase"]
+        self.assertEqual(codebase, "A")
+        
+    def test_properties_repository(self):
+        self.build.setupProperties()
+        repository = self.props["Build"]["repository"]
+        self.assertEqual(repository, "http://svn-repo-A")
+        
+    def test_properties_revision(self):
+        self.build.setupProperties()
+        revision = self.props["Build"]["revision"]
+        self.assertEqual(revision, "12345")
+        
+    def test_properties_branch(self):
+        self.build.setupProperties()
+        branch = self.props["Build"]["branch"]
+        self.assertEqual(branch, "develop")
+
+    def test_property_project(self):
+        self.build.setupProperties()
+        project = self.props["Build"]["project"]
+        self.assertEqual(project, '')
+        
 class TestBuildProperties(unittest.TestCase):
     """
     Test that a Build has the necessary L{IProperties} methods, and that they
@@ -483,6 +739,9 @@ class TestBuildProperties(unittest.TestCase):
 
     def setUp(self):
         r = FakeRequest()
+        r.sources = [FakeSource()]
+        r.sources[0].changes = [FakeChange()]
+        r.sources[0].revision = "12345"
         self.build = Build([r])
         self.build.setStepFactories([])
         self.builder = Mock()
